﻿module coneneko.rp2;
private import std.cstream;
private import nanami.camera;

template Rp2()
{
	void scene2()
	{
		Model model = new Model();
		addControl(new OutputSlider(-0.1, -0.93, 0.9, 0.04, &model.sliderX));
		addControl(new InputSlider(0.9, 0.9, 0.04, 0.9, &model.sliderY));
		addControl(new Button(0.9, -0.9, 0.05, 0.05, &model.pushButton));
		addControl(new CheckButton(0.7, -0.3, 0.1, 0.1, &model.checkButton));
		addControl(new CheckButton(0.85, -0.3, 0.1, 0.1, &model.checkButton));
		addControl(new CheckButton(0.7, -0.45, 0.1, 0.1, &model.checkButton));
		addControl(new CheckButton(0.85, -0.45, 0.1, 0.1, &model.checkButton));
		RadioButtonInfo rbInfo = new RadioButtonInfo();
		rbInfo.addButton(0.7, -0.1, 0.1, 0.1);
		rbInfo.addButton(0.85, -0.1, 0.1, 0.1);
		addControl(new RadioButton(rbInfo, &model.selectButton));
		
		CameraEx camera = new CameraEx();
		
		while (!model.eof)
		{
			foreach (uint input; this) camera.input(input);
			camera.tick(mouseX, mouseY);
			Matrix modelView = Matrix.scaling(0.005, 0.005, 0.005)
				* camera.getViewMatrix();
			
			model.tickSlider();
			drawModel(`..\resource2\test0.773`, model.motionIndex, modelView,
				model.motionSpeed);
			update();
			sleep(10);
		}
		clearControl();
	}
	
	void drawModel(char[] fileName, uint motionIndex, Matrix modelView,
		uint motionSpeed)
	{
		if (!has(fileName)) addNanami(fileName);
		
		depthBuffer = true;
		
		MotionIterator i = getMotionIterator(fileName);
		i.motionIndex = motionIndex;
		for (int j = 0; j < motionSpeed; j++) i.moveNext();
		if (i.isEnd()) i.moveBegin();
		
		setToonShaderLightDirection(Vector.create(0, -1, 0));
		setModelViewMatrix(modelView);
		setProjectionMatrix(Matrix.perspectiveFovRH(PI / 3.0, 640.0 / 480.0, 1.0, 1000.0));
		drawNanami(fileName);
	}
}

class Model
{
	private float _sliderX = 0.0;
	float sliderX() { return _sliderX; }
	void sliderX(float a) { _sliderX = a; }
	
	void sliderY(float a)
	{
		dout.writefln("slider %f ", a);
		if (a < 0.1)
		{
			_motionIndex = 0;
			_motionSpeed = 1;
		}
		else
		{
			_motionIndex = 1;
			_motionSpeed = 1 + cast(uint)(a * 3);
		}
	}
	
	void pushButton()
	{
		dout.writeLine("push button");
		_eof = true;
	}
	
	void checkButton(bit flag)
	{
		dout.writeLine(flag ? "push check button" : "pop check button");
	}
	
	void selectButton(uint index)
	{
		dout.writefln("select button %d", index);
	}
	
	private bit _eof = false;
	bit eof() { return _eof; }
	
	private uint _motionIndex = 0;
	uint motionIndex()
	{
		return toRight ? _motionIndex : 0;
	}
	
	private uint _motionSpeed = 1;
	uint motionSpeed()
	{
		return toRight ? _motionSpeed : 1;
	}
	
	private bit toRight = true;
	void tickSlider()
	{
		if (toRight)
		{
			_sliderX += 0.001 * motionSpeed;
			if (_sliderX > 1.0)
			{
				_sliderX = 1.0;
				toRight = false;
			}
		}
		else
		{
			_sliderX -= 0.001;
			if (_sliderX < 0.0)
			{
				_sliderX = 0.0;
				toRight = true;
			}
		}
	}
}

class CameraEx : Camera
{
	void input(uint n)
	{
		switch (n)
		{
			case 0: pushLeft(); break;
			case 1: popLeft(); break;
			case 2: pushRight(); break;
			case 3: popRight(); break;
			default: break;
		}
	}
}
