import std.cstream;
import std.string;

// base, drawCommandList, onceCommandList
int main(char[][] args)
{
	const char[] T1 = "\t", T2 = "\t\t", T3 = "\t\t\t", T4 = "\t\t\t\t";
	bit once = args.length == 2 && args[1] == "-once";
	
	bit decoration = args.length == 2 && args[1] == "-decoration";
	bit staticDecoration = args.length == 3 && args[1] == "-static_decoration";
	if (decoration || staticDecoration)
	{
		char[] superClass;
		if (staticDecoration) superClass = args[2];
		generateDecoration(staticDecoration, superClass);
		return 0;
	}
	
	dout.writeLine("");
	dout.writefln("template %sCommands()", once ? "Once" : "Draw");
	dout.writeLine("{");
	while (true)
	{
		char[] text = din.readLine();
		if (!text) break;
		dout.writeLine("");
		Function f = new Function(text);
		dout.writeLine(T1 ~ f.text2);
		dout.writeLine(T1 ~ "{");
		dout.writefln(T2 ~ "class %sCommand : Command", f.name2);
		dout.writeLine(T2 ~ "{");
		dout.writeLine(T3 ~ "private final WindowResource wr;");
		for (int i = 0; i < f.argLength; i++)
		{
			dout.writefln(T3 ~ "private final %s %s;", f.getType(i), f.getName(i));
		}
		dout.writeLine("");
		dout.writefln(T3 ~ "this(WindowResource wr%s)",
			f.argLength != 0 ? ", " ~ f.args : "");
		dout.writeLine(T3 ~ "{");
		dout.writeLine(T4 ~ "this.wr = wr;");
		for (int i = 0; i < f.argLength; i++)
		{
			dout.writefln(T4 ~ "this.%s = %s;", f.getName(i), f.getName(i));
		}
		dout.writeLine(T3 ~ "}");
		dout.writeLine("");
		dout.writeLine(T3 ~ "void opCall()");
		dout.writeLine(T3 ~ "{");
		dout.writefln(T4 ~ "wr.%s", f.subType());
		dout.writeLine(T3 ~ "}");
		dout.writeLine(T2 ~ "}");
		dout.writeLine("");
		if (once)
		{
			dout.writefln(T2 ~ "onceCommandList ~= new %sCommand(base%s);",
				f.name2, f.argLength != 0 ? ", " ~ f.inner : "");
		}
		else
		{
			dout.writefln(T2 ~ "nextDrawCommandList ~= new %sCommand(base%s);",
				f.name2, f.argLength != 0 ? ", " ~ f.inner : "");
		}
		dout.writeLine(T1 ~ "}");
	}
	dout.writeLine("}");
	return 0;
}

class Function
{
	unittest
	{
		Function f0 = new Function("void f0();");
		assert("f0" == f0.name);
		assert(0 == f0.argLength);
		assert("F0" == f0.name2);
		assert("" == f0.args);
		assert("f0();" == f0.subType());
		assert("void f0()" == f0.text2);
		assert("" == f0.inner);
		
		Function f1 = new Function("void f1(int n);");
		assert("f1" == f1.name);
		assert(1 == f1.argLength);
		assert("F1" == f1.name2);
		assert("int n" == f1.args);
		assert("int n" == f1.splitArgs()[0]);
		assert("int" == f1.getType(0));
		assert("n" == f1.getName(0));
		assert("f1(n);" == f1.subType());
		assert("void f1(int n)" == f1.text2);
		assert("n" == f1.inner);
		
		Function f2 = new Function("void f2(char[] a, int n);");
		assert("f2" == f2.name);
		assert(2 == f2.argLength);
		assert("F2" == f2.name2);
		assert("char[] a, int n" == f2.args);
		assert("char[] a" == f2.splitArgs()[0]);
		assert("int n" == f2.splitArgs()[1]);
		assert("char[]" == f2.getType(0));
		assert("int" == f2.getType(1));
		assert("a" == f2.getName(0));
		assert("n" == f2.getName(1));
		assert("f2(a, n);" == f2.subType());
		assert("void f2(char[] a, int n)" == f2.text2);
		assert("a, n" == f2.inner);
		
		Function f3 = new Function("dchar[][] f3();");
		assert("f3" == f3.name);
		assert(0 == f3.argLength);
		
		Function f4 = new Function("void f4(out uint a);");
		assert("f4" == f4.name);
		assert(1 == f4.argLength);
		assert("out uint a" == f4.splitArgs()[0]);
		assert("a" == f4.getName(0));
		
		Function f5 = new Function("void f5(dchar[] abc = null);");
		assert("f5" == f5.name);
		assert(1 == f5.argLength);
		assert("dchar[] abc = null" == f5.splitArgs()[0]);
		assert("abc" == f5.getName(0));
	}
	
	private final char[] text;
	
	this(char[] text)
	{
		this.text = text;
	}
	
	char[] name()
	{
		int begin = find(text, ' ') + 1;
		int end = find(text, '(');
		return text[begin..end];
	}
	
	uint argLength()
	{
		if (-1 != find(text, "()")) return 0;
		return 1 + count(text, ",");
	}
	
	char[] name2()
	{
		return toupper(name[0..1]) ~ name[1..$];
	}
	
	char[] args()
	{
		int begin = find(text, "(") + 1;
		int end = find(text, ")");
		return text[begin..end];
	}
	
	char[][] splitArgs()
	{
		char[][] result = split(args, ",");
		foreach (inout char[] a; result) a = stripl(a);
		return result;
	}
	
	char[] getType(int index)
	in
	{
		assert(index < argLength);
	}
	body
	{
		return split(splitArgs()[index])[0];
	}
	
	char[] getName(int index)
	in
	{
		assert(index < argLength);
	}
	body
	{
		char[][] list = split(splitArgs()[index]);
		return list[0] == "out" ? list[2] : list[1];
	}
	
	char[] subType()
	{
		return format("%s(%s);", name, inner);
	}
	
	char[] text2()
	{
		return text[0..$ - 1];
	}
	
	char[] inner()
	{
		char[] result;
		for (int i = 0; i < argLength; i++)
		{
			result ~= getName(i);
			if (i + 1 < argLength) result ~= ", ";
		}
		return result;
	}
}

void generateDecoration(bit staticDecoration, char[] superClass)
{
	const char[] T1 = "\t", T2 = "\t\t", T3 = "\t\t\t", T4 = "\t\t\t\t";
	if (superClass == "") superClass = "base";
	
	dout.writeLine("");
	dout.writefln("template %s()",
		staticDecoration ? "StaticDecoration" : "Decoration");
	dout.writeLine("{");
	while (true)
	{
		char[] text = din.readLine();
		if (!text) break;
		dout.writeLine("");
		Function f = new Function(text);
		dout.writeString(T1);
		if (staticDecoration) dout.writeString("static ");
		dout.writeLine(f.text2);
		dout.writeLine(T1 ~ "{");
		dout.writefln(T2 ~ "return %s.%s", superClass, f.subType());
		dout.writeLine(T1 ~ "}");
	}
	dout.writeLine("}");
}
