﻿module hdk.error;

class HdkError : Error
{
	this(char[] msg) { super(msg); }
}

template HdkError_()
{
	private import std.string;
	static assert(__LINE__.max == long.max);
	protected static void error(long line, char[] msg = "")
	{
		throw new HdkError(format("%s %s %d", this.classinfo.name, msg, line));
	}
	protected static void error(char[] msg)
	{
		throw new HdkError(msg);
	}
}

template SdlError()
{
	version (hdk_lib) private import sdl;
	mixin HdkError_;
	//invariant { throwIfSdlError(__LINE__); }
	protected void throwIfSdlError(long line)
	{
		char[] errorMessage = toString(SDL_GetError());
		if (errorMessage.length != 0) error(line, errorMessage);
	}
}

template OglError()
{
	private import opengl;
	mixin HdkError_;
	//invariant { throwIfOglError(__LINE__); } // 初期化していないスレッドで使うとエラーになる
	protected void throwIfOglError(long line)
	{
		switch (glGetError())
		{
			case GL_NO_ERROR:          return;
			case GL_INVALID_ENUM:      error(line, "GL_INVALID_ENUM");
			case GL_INVALID_VALUE:     error(line, "GL_INVALID_VALUE");
			case GL_INVALID_OPERATION: error(line, "GL_INVALID_OPERATION");
			case GL_STACK_OVERFLOW:    error(line, "GL_STACK_OVERFLOW");
			case GL_STACK_UNDERFLOW:   error(line, "GL_STACK_UNDERFLOW");
			case GL_OUT_OF_MEMORY:     error(line, "GL_OUT_OF_MEMORY");
			default:                   error(line, "OpenGL unknown error");
		}
	}
}
