﻿module hdk.oglbase;

template OglBase()
{
	private import hdk.openglarb;
	private import hdk.error;
	mixin OglError;
	
	private static bit initialized = false;
	
	this()
	{
		if (initialized) error(__LINE__, "既に初期化されてる");
		initialized = true;
		
		initializeArb();
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	
	~this() { initialized = false; }
	
	void clear()
	{
		depthTestWriting(true, true);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	}
	
	void drawArrays(uint vertexLength) { glDrawArrays(GL_TRIANGLES, 0, vertexLength); }
	
	private static void swap(inout uint a, inout uint b)
	{
		uint tmp = a;
		a = b;
		b = tmp;
	}
	
	void setProjectionMatrix(float* f4x4)
	{
		glMatrixMode(GL_PROJECTION);
		glLoadMatrixf(f4x4);
	}
	
	void setModelViewMatrix(float* f4x4)
	{
		glMatrixMode(GL_MODELVIEW);
		glLoadMatrixf(f4x4);
	}
	
	void projection(Matrix m) { setProjectionMatrix(cast(float*)&m); }
	void modelView(Matrix m) { setModelViewMatrix(cast(float*)&m); }
	
	void depthTestWriting(bit test, bit writing)
	{
		test ? glEnable(GL_DEPTH_TEST) : glDisable(GL_DEPTH_TEST);
		glDepthMask(cast(GLboolean)(writing ? GL_TRUE : GL_FALSE));
	}
	
	void cullingEnableFront(bit enable, bit front)
	{
		enable ? glEnable(GL_CULL_FACE) : glDisable(GL_CULL_FACE);
		glCullFace(front ? GL_FRONT : GL_BACK);
	}
	
	uint[] readBackBuffer(uint width, uint height)
	{
		uint[] pixels = new uint[width * height];
		glReadBuffer(GL_BACK);
		glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, pixels.ptr);
		
		uint[] result = new uint[width * height];
		for (int y = 0; y < height; y++)
		{
			int y2 = (height - 1) - y;
			int begin = width * y;
			int end = begin + width;
			int begin2 = width * y2;
			int end2 = begin2 + width;
			result[begin..end] = pixels[begin2..end2].dup;
		}
		delete pixels;
		
		const uint ALPHA_FILTER = 0xff000000;
		foreach (inout uint a; result) { a |= ALPHA_FILTER; }
		
		return result;
	}
}
