﻿module hdk.openglarb;
private import std.string;
private import opengl;
private import sdl;

alias char GLcharARB;
alias int GLhandleARB;
static assert(int.max == GLint.max);
static assert(uint.max == GLenum.max);

enum : GLenum
{
	GL_ARRAY_BUFFER_ARB = 0x8892,
	GL_ELEMENT_ARRAY_BUFFER_ARB = 0x8893,
	GL_STATIC_DRAW_ARB = 0x88E4,
	
	GL_VERTEX_SHADER_ARB = 0x8B31,
	GL_FRAGMENT_SHADER_ARB = 0x8B30,
	GL_OBJECT_COMPILE_STATUS_ARB = 0x8B81,
	GL_OBJECT_LINK_STATUS_ARB = 0x8B82,
	GL_OBJECT_INFO_LOG_LENGTH_ARB = 0x8B84,
	
	GL_TEXTURE0_ARB = 0x84C0,
	GL_TEXTURE1_ARB = 0x84C1,
	GL_TEXTURE2_ARB = 0x84C2,
	GL_TEXTURE3_ARB = 0x84C3,
}

extern (Windows) // TODO linuxとの互換性
{
	void function(GLenum target, GLuint buffer) glBindBufferARB;
	void function(GLsizei n, GLuint* buffers) glDeleteBuffersARB;
	void function(GLsizei n, GLuint* buffers) glGenBuffersARB;
	void function(GLenum target, int size, GLvoid* data, GLenum usage) glBufferDataARB;
	
	void function(GLuint index) glEnableVertexAttribArrayARB;
	void function(GLuint index) glDisableVertexAttribArrayARB;
	void function(GLuint index, GLint size, GLenum type,
		GLboolean normalized, GLsizei stride, GLvoid *pointer) glVertexAttribPointerARB;
	
	void function(GLhandleARB obj) glDeleteObjectARB;
	void function(GLhandleARB programObj) glUseProgramObjectARB;
	void function(GLhandleARB obj, GLenum pname, GLint *params) glGetObjectParameterivARB;
	void function(GLhandleARB obj,GLsizei maxLength, GLsizei *length, GLcharARB *infoLog) glGetInfoLogARB;
	GLhandleARB function() glCreateProgramObjectARB;
	GLhandleARB function(GLenum shaderType) glCreateShaderObjectARB;
	void function(GLhandleARB shaderObj, GLsizei count, GLcharARB **strings, GLint *length) glShaderSourceARB;
	void function(GLhandleARB shaderObj) glCompileShaderARB;
	void function(GLhandleARB containerObj, GLhandleARB attachedObj) glDetachObjectARB;
	void function(GLhandleARB containerObj, GLhandleARB obj) glAttachObjectARB;
	void function(GLhandleARB programObj) glLinkProgramARB;
	
	GLint function(GLhandleARB programObj, GLcharARB *name) glGetUniformLocationARB;
	void function(GLint location, GLfloat v0) glUniform1fARB;
	void function(GLint location, GLint v0) glUniform1iARB;
	void function(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3) glUniform4fARB;
	void function(GLint location, GLsizei count, GLboolean transpose, GLfloat *value) glUniformMatrix4fvARB;
	
	GLint function(GLhandleARB program, GLcharARB *name) glGetAttribLocationARB;
	
	void function(GLenum) glActiveTextureARB;
	void function(GLenum, GLfloat, GLfloat) glMultiTexCoord2fARB;
	void function(GLenum, GLfloat *) glMultiTexCoord2fvARB;
	void function(GLenum) glClientActiveTextureARB;
	
	void function(GLuint index, GLfloat *v) glVertexAttrib2fvARB;
	void function(GLuint index, GLfloat *v) glVertexAttrib1fvARB;
}

private bit initialized = false;

void initializeArb() // SDL初期化後に
{
	void load(void** f, char[] name)
	{
		if (*f == null) *f = SDL_GL_GetProcAddress(toStringz(name));
		if (*f == null) throw new Error(name ~ " is empty");
	}
	
	if (initialized) return;
	
	load(cast(void**)&glBindBufferARB, "glBindBufferARB");
	load(cast(void**)&glDeleteBuffersARB, "glDeleteBuffersARB");
	load(cast(void**)&glGenBuffersARB, "glGenBuffersARB");
	load(cast(void**)&glBufferDataARB, "glBufferDataARB");
	
	load(cast(void**)&glEnableVertexAttribArrayARB, "glEnableVertexAttribArrayARB");
	load(cast(void**)&glDisableVertexAttribArrayARB, "glDisableVertexAttribArrayARB");
	load(cast(void**)&glVertexAttribPointerARB, "glVertexAttribPointerARB");
	
	load(cast(void**)&glGetObjectParameterivARB, "glGetObjectParameterivARB");
	load(cast(void**)&glGetInfoLogARB, "glGetInfoLogARB");
	load(cast(void**)&glCreateProgramObjectARB, "glCreateProgramObjectARB");
	load(cast(void**)&glCreateShaderObjectARB, "glCreateShaderObjectARB");
	load(cast(void**)&glShaderSourceARB, "glShaderSourceARB");
	load(cast(void**)&glCompileShaderARB, "glCompileShaderARB");
	load(cast(void**)&glDetachObjectARB, "glDetachObjectARB");
	load(cast(void**)&glAttachObjectARB, "glAttachObjectARB");
	load(cast(void**)&glLinkProgramARB, "glLinkProgramARB");
	load(cast(void**)&glUseProgramObjectARB, "glUseProgramObjectARB");
	load(cast(void**)&glDeleteObjectARB, "glDeleteObjectARB");
	
	load(cast(void**)&glGetUniformLocationARB, "glGetUniformLocationARB");
	load(cast(void**)&glUniform1fARB, "glUniform1fARB");
	load(cast(void**)&glUniform1iARB, "glUniform1iARB");
	load(cast(void**)&glUniform4fARB, "glUniform4fARB");
	load(cast(void**)&glUniformMatrix4fvARB, "glUniformMatrix4fvARB");
	
	load(cast(void**)&glGetAttribLocationARB, "glGetAttribLocationARB");
	
	load(cast(void**)&glActiveTextureARB, "glActiveTextureARB");
	load(cast(void**)&glMultiTexCoord2fARB, "glMultiTexCoord2fARB");
	load(cast(void**)&glMultiTexCoord2fvARB, "glMultiTexCoord2fvARB");
	load(cast(void**)&glClientActiveTextureARB, "glClientActiveTextureARB");
	
	load(cast(void**)&glVertexAttrib2fvARB, "glVertexAttrib2fvARB");
	load(cast(void**)&glVertexAttrib1fvARB, "glVertexAttrib1fvARB");
	
	
	load(cast(void**)&glGetBufferParameterivARB, "glGetBufferParameterivARB");
	load(cast(void**)&glGetBufferSubDataARB, "glGetBufferSubDataARB");
	load(cast(void**)&glGetUniformfvARB, "glGetUniformfvARB");
	load(cast(void**)&glGetUniformivARB, "glGetUniformivARB");
	
	initialized = true;
}




enum : GLenum { GL_BUFFER_SIZE_ARB = 0x8764 }

extern (Windows)
{
	typedef ptrdiff_t GLintptrARB;
	typedef ptrdiff_t GLsizeiptrARB;
	void function(GLenum, GLenum, GLint *) glGetBufferParameterivARB;
	void function(GLenum, GLintptrARB, GLsizeiptrARB, GLvoid *) glGetBufferSubDataARB;
	void function(GLhandleARB program, GLint location, GLfloat *params) glGetUniformfvARB;
	void function(GLhandleARB program, GLint location, GLint *params) glGetUniformivARB;
}
