﻿import std.cstream;
import hdk.window;
import hdk.zipex;
import std.file;
import hdk.sdlimage;

int main()
{
	try
	{
		const char[] TEST_DATA_ZIP = "test9data.zip";
		
		if (!exists(TEST_DATA_ZIP))
		{
			ZipArchiveEx encoder = new ZipArchiveEx();
			uint[] texture = new uint[256 * 256];
			for (int i = 0; i < texture.length; i++)
			{
				uint x = i % 256;
				uint y = i / 256;
				texture[i] = 0xff000000 | (y << 8) | (x);
			}
			encoder.addTexture("texture", texture, 256, 256);
			
			StripGenerator sg = new StripGenerator();
			float[] vb = sg.createVertexBuffer();
			int sizeOfVertex = (3 + 4 + 3 + 2 + 2 + 1); // pcntmw
			int vertexLength = vb.length / sizeOfVertex;
			encoder.addVertexBuffer("vertex_buffer", vb, sizeOfVertex, vertexLength);
			
			write(TEST_DATA_ZIP, encoder.build());
			dout.writeLine("created " ~ TEST_DATA_ZIP);
		}
		
		const dchar[] TEXTURE = `test9data.zip\texture`;
		//const dchar[] TEXTURE = `..\resource\test1.png`;
		
		alias Vector.create rgba;
		auto Window wnd = new Window();
		wnd.postTerminableCommand(new DrawBlendVertex());
		wnd.drawTexture(TEXTURE, rgba(1, 1, 1, 0.5));
		wnd.update();
		
		sleep(100);
		uint[] backBuffer = wnd.readBackBuffer();
		assert(wnd.width * wnd.height == backBuffer.length);
		writeBitmap("test9.bmp", backBuffer, wnd.width, wnd.height);
		
		wnd.wait();
	}
	catch (Exception e)
	{
		derr.writeLine("Exception : " ~ e.toString());
	}
	return 0;
}

class DrawBlendVertex : TerminableCommand
{
	bit alive() { return true; }
	
	private float f = 0.0;
	
	void opCall(WindowResource wr)
	{
		const dchar[] VERTEX = `test9data.zip\vertex_buffer`;
		const dchar[] VERTEX_BLEND_SHADER = "vertex_blend";
		
		f -= 0.01;
		
		Matrix[] matrix = new Matrix[2]; // Matrix[2] matrix;では動かない
		matrix[0] = Matrix.identity;
		matrix[1] = Matrix.rotationZ(f);
		wr.setShaderMatrixArray(VERTEX_BLEND_SHADER, "matrixArray", matrix);
		
		wr.drawPcntmw(VERTEX_BLEND_SHADER, "matrixIndex", "weight", VERTEX, null);
	}
}

class StripGenerator
{
	unittest
	{
		const float[] list = [ 0, 0, 0, 1, 1, 1, 2, 2, 2, 3, 3, 3 ];
		const float[] result = [ 0, 0, 0, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 3, 3, 3 ];
		//foreach (float a; stripToTriangle(list)) dout.writef("%f ", a);
		assert(result == stripToTriangle(list));
	}
	
	private static float[] stripToTriangle(float[] list)
	in
	{
		assert(9 < list.length);
	}
	out (result)
	{
		assert(0 == result.length % 9);
	}
	body
	{
		float[] result;
		for (int i = 0; i <= list.length - 9; i += 3)
		{
			result ~= list[i..i + 3];
			result ~= list[i + 3..i + 6];
			result ~= list[i + 6..i + 9];
		}
		return result;
	}
	
	private float[] positionList()
	{
		float[] result;
		for (int i = 0; i < 100; i++)
		{
			result ~= vec3(0.01, 0.0 + 0.01 * i, 0);
			result ~= vec3(-0.01, 0.005 + 0.01 * i, 0);
		}
		return stripToTriangle(result);
	}
	
	private float[] weightList(float[] positionList)
	{
		float[] result = new float[1 * vertexBufferLength];
		for (int i = 0; i < result.length; i++)
		{
			result[i] = 1.0 - positionList[3 * i + 1];
			assert(0 <= result[i] && result[i] <= 1.0);
		}
		return result;
	}
	
	float[] createVertexBuffer()
	{
		float[] result;
		float[] pl = positionList();
		result ~= pl;
		result ~= createCntmVertexBuffer();
		result ~= weightList(pl);
		return result;
	}
	
	private uint vertexBufferLength()
	out (result)
	{
		assert(0 == result % 3);
	}
	body
	{
		return positionList.length / 3;
	}
	
	private float[] createCntmVertexBuffer()
	{
		float[] result;
		
		float[] color = new float[4 * vertexBufferLength];
		color[] = 1.0;
		result ~= color;
		
		float[] normal = new float[3 * vertexBufferLength];
		normal[] = 0.0;
		result ~= normal;
		
		float[] texCoord = new float[2 * vertexBufferLength];
		texCoord[] = 0.0;
		result ~= texCoord;
		
		float[] matrixIndex;
		for (int i = 0; i < vertexBufferLength; i++)
		{
			matrixIndex ~= vec2(0, 1);
		}
		result ~= matrixIndex;
		
		return result;
	}
	
	private static float[] vec3(float x, float y, float z)
	out (result)
	{
		assert(3 == result.length);
	}
	body
	{
		float[] result = new float[3];
		result[0] = x;
		result[1] = y;
		result[2] = z;
		return result;
	}
	
	private static float[] vec2(float x, float y)
	out (result)
	{
		assert(2 == result.length);
	}
	body
	{
		float[] result = new float[2];
		result[0] = x;
		result[1] = y;
		return result;
	}
}
